// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:nertc_core_platform_interface/nertc_core_platform_interface.dart';
import 'package:nertc_core_platform_interface/pigeon.dart';

class MethodChannelNERtcAudioEffect extends NERtcAudioEffectPlatform {
  final _api = AudioEffectApi();
  final _eventDispatcher = MethodChannelAudioEffectEventSink();

  MethodChannelNERtcAudioEffect() {
    NERtcAudioEffectEventSink.setup(_eventDispatcher);
  }

  @override
  void addRtcAudioEffectListener(NERtcAudioEffectEventSink sink) {
    _eventDispatcher.addRtcAudioEffectListener(sink);
  }

  @override
  void removeRtcAudioEffectListener(NERtcAudioEffectEventSink sink) {
    _eventDispatcher.removeRtcAudioEffectListener(sink);
  }

  @override
  Future<int> playEffect(int effectId, NERtcAudioEffectOptions options) async {
    int reply = await _api.playEffect(PlayEffectRequest()
      ..effectId = effectId
      ..path = options.path
      ..loopCount = options.loopCount
      ..sendEnabled = options.sendEnabled
      ..sendVolume = options.sendVolume
      ..playbackEnabled = options.playbackEnabled
      ..playbackVolume = options.playbackVolume
      ..startTimestamp = options.startTimestamp
      ..sendWithAudioType = options.sendWithAudioType
      ..progressInterval = options.progressInterval);
    return reply;
  }

  @override
  Future<int> stopEffect(int effectId) async {
    int reply = await _api.stopEffect(effectId);
    return reply;
  }

  @override
  Future<int> stopAllEffects() async {
    int reply = await _api.stopAllEffects();
    return reply;
  }

  @override
  Future<int> pauseAllEffect() async {
    int reply = await _api.pauseAllEffects();
    return reply;
  }

  @override
  Future<int> pauseEffect(int effectId) async {
    int reply = await _api.pauseEffect(effectId);
    return reply;
  }

  @override
  Future<int> resumeAllEffect() async {
    int reply = await _api.resumeAllEffects();
    return reply;
  }

  @override
  Future<int> resumeEffect(int effectId) async {
    int reply = await _api.resumeEffect(effectId);
    return reply;
  }

  @override
  Future<int> getEffectCurrentPosition(int effectId) async {
    int reply = await _api.getEffectCurrentPosition(effectId);
    return reply;
  }

  @override
  Future<int> getEffectDuration(int effectId) async {
    int reply = await _api.getEffectDuration(effectId);
    return reply;
  }

  @override
  Future<int> getEffectSendVolume(int effectId) async {
    int reply = await _api.getEffectSendVolume(effectId);
    return reply;
  }

  @override
  Future<int> setEffectPlaybackVolume(int effectId, int volume) async {
    int reply = await _api.setEffectPlaybackVolume(effectId, volume);
    return reply;
  }

  @override
  Future<int> setEffectSendVolume(int effectId, int volume) async {
    int reply = await _api.setEffectSendVolume(effectId, volume);
    return reply;
  }

  @override
  Future<int> getEffectPitch(int effectId) async {
    int reply = await _api.getEffectPitch(effectId);
    return reply;
  }

  @override
  Future<int> getEffectPlaybackVolume(int effectId) async {
    int reply = await _api.getEffectPlaybackVolume(effectId);
    return reply;
  }

  @override
  Future<int> setEffectPitch(int effectId, int pitch) async {
    int reply = await _api.setEffectPitch(effectId, pitch);
    return reply;
  }

  @override
  Future<int> setEffectPosition(int effectId, int position) async {
    int reply = await _api.setEffectPosition(effectId, position);
    return reply;
  }
}

class MethodChannelAudioEffectEventSink extends NERtcAudioEffectEventSink {
  final _listeners = <NERtcAudioEffectEventSink>[];

  void addRtcAudioEffectListener(NERtcAudioEffectEventSink sink) {
    _listeners.add(sink);
  }

  void removeRtcAudioEffectListener(NERtcAudioEffectEventSink sink) {
    _listeners.remove(sink);
  }

  @override
  void onAudioEffectFinished(int effectId) {
    for (var element in _listeners) {
      element.onAudioEffectFinished(effectId);
    }
  }

  @override
  void onAudioEffectTimestampUpdate(int id, int timestampMs) {
    for (var element in _listeners) {
      element.onAudioEffectTimestampUpdate(id, timestampMs);
    }
  }
}
