// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:nertc_core_platform_interface/nertc_core_platform_interface.dart';
import 'package:nertc_core_platform_interface/pigeon.dart';

class MethodChannelNERtcAudioMixing extends NERtcAudioMixingPlatform {
  final _api = AudioMixingApi();
  final _eventDispatcher = MethodChannelRtcAudioMixingEventSink();

  MethodChannelNERtcAudioMixing() {
    NERtcAudioMixingEventSink.setup(_eventDispatcher);
  }

  @override
  void addRtcAudioMixingListener(NERtcAudioMixingEventSink sink) {
    _eventDispatcher.addRtcAudioMixingListener(sink);
  }

  @override
  void removeRtcAudioMixingListener(NERtcAudioMixingEventSink sink) {
    _eventDispatcher.removeRtcAudioMixingListener(sink);
  }

  @override
  Future<int> startAudioMixing(NERtcAudioMixingOptions options) async {
    int reply = await _api.startAudioMixing(StartAudioMixingRequest(
        path: options.path,
        loopCount: options.loopCount,
        sendEnabled: options.sendEnabled,
        sendVolume: options.sendVolume,
        playbackEnabled: options.playbackEnabled,
        playbackVolume: options.playbackVolume,
        startTimeStamp: options.startTimeStamp,
        sendWithAudioType: options.sendWithAudioType,
        progressInterval: options.progressInterval));
    return reply;
  }

  @override
  Future<int> stopAudioMixing() async {
    int reply = await _api.stopAudioMixing();
    return reply;
  }

  @override
  Future<int> getAudioMixingCurrentPosition() async {
    int reply = await _api.getAudioMixingCurrentPosition();
    return reply;
  }

  @override
  Future<int> getAudioMixingDuration() async {
    int reply = await _api.getAudioMixingDuration();
    return reply;
  }

  @override
  Future<int> getAudioMixingPitch() async {
    int reply = await _api.getAudioMixingPitch();
    return reply;
  }

  @override
  Future<int> getAudioMixingPlaybackVolume() async {
    int reply = await _api.getAudioMixingPlaybackVolume();
    return reply;
  }

  @override
  Future<int> getAudioMixingSendVolume() async {
    int reply = await _api.getAudioMixingSendVolume();
    return reply;
  }

  @override
  Future<int> pauseAudioMixing() async {
    int reply = await _api.pauseAudioMixing();
    return reply;
  }

  @override
  Future<int> resumeAudioMixing() async {
    int reply = await _api.resumeAudioMixing();
    return reply;
  }

  @override
  Future<int> setAudioMixingPitch(int pitch) async {
    int reply = await _api.setAudioMixingPitch(pitch);
    return reply;
  }

  @override
  Future<int> setAudioMixingPlaybackVolume(int volume) async {
    int reply = await _api.setAudioMixingPlaybackVolume(volume);
    return reply;
  }

  @override
  Future<int> setAudioMixingPosition(int position) async {
    int reply = await _api.setAudioMixingPosition(position);
    return reply;
  }

  @override
  Future<int> setAudioMixingSendVolume(int volume) async {
    int reply = await _api.setAudioMixingSendVolume(volume);
    return reply;
  }
}

class MethodChannelRtcAudioMixingEventSink extends NERtcAudioMixingEventSink {
  final _listeners = <NERtcAudioMixingEventSink>[];

  void addRtcAudioMixingListener(NERtcAudioMixingEventSink sink) {
    _listeners.add(sink);
  }

  void removeRtcAudioMixingListener(NERtcAudioMixingEventSink sink) {
    _listeners.remove(sink);
  }

  @override
  void onAudioMixingStateChanged(int reason) {
    for (var element in _listeners) {
      element.onAudioMixingStateChanged(reason);
    }
  }

  @override
  void onAudioMixingTimestampUpdate(int timestampMs) {
    for (var element in _listeners) {
      element.onAudioMixingTimestampUpdate(timestampMs);
    }
  }
}
