// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'dart:async';
import 'package:nertc_core_platform_interface/nertc_core_platform_interface.dart';
import 'package:nertc_core_platform_interface/pigeon.dart';

class MethodChannelNERtcDevice extends NERtcDevicePlatform {
  final _api = DeviceManagerApi();
  final _eventDispatcher = MethodChannelRtcDeviceEventSink();

  MethodChannelNERtcDevice() {
    NERtcDeviceEventSink.setup(_eventDispatcher);
  }

  void addRtcDeviceListener(NERtcDeviceEventSink sink) async {
    _eventDispatcher.addRtcDeviceListener(sink);
  }

  void removeRtcDeviceListener(NERtcDeviceEventSink sink) async {
    _eventDispatcher.removeRtcDeviceListener(sink);
  }

  @override
  Future<double> getCameraMaxZoom() async {
    double reply = await _api.getCameraMaxZoom();
    return reply;
  }

  @override
  Future<int> setCameraZoomFactor(double factor) async {
    int reply = await _api.setCameraZoomFactor(factor);
    return reply;
  }

  @override
  Future<int> switchCamera() async {
    int reply = await _api.switchCamera();
    return reply;
  }

  @override
  Future<int> setCameraFocusPosition(double x, double y) async {
    int reply = await _api.setCameraFocusPosition(SetCameraPositionRequest()
      ..x = x
      ..y = y);
    return reply;
  }

  @override
  Future<int> setCameraTorchOn(bool on) async {
    int reply = await _api.setCameraTorchOn(on);
    return reply;
  }

  @override
  Future<bool> isCameraExposurePositionSupported() async {
    bool reply = await _api.isCameraExposurePositionSupported();
    return reply;
  }

  @override
  Future<bool> isCameraFocusSupported() async {
    bool reply = await _api.isCameraFocusSupported();
    return reply;
  }

  @override
  Future<bool> isCameraTorchSupported() async {
    bool reply = await _api.isCameraTorchSupported();
    return reply;
  }

  @override
  Future<bool> isCameraZoomSupported() async {
    bool reply = await _api.isCameraZoomSupported();
    return reply;
  }

  @override
  Future<bool> isPlayoutDeviceMute() async {
    bool reply = await _api.isPlayoutDeviceMute();
    return reply;
  }

  @override
  Future<bool> isRecordDeviceMute() async {
    bool reply = await _api.isRecordDeviceMute();
    return reply;
  }

  @override
  Future<int> setPlayoutDeviceMute(bool mute) async {
    int reply = await _api.setPlayoutDeviceMute(mute);
    return reply;
  }

  @override
  Future<int> setRecordDeviceMute(bool mute) async {
    int reply = await _api.setRecordDeviceMute(mute);
    return reply;
  }

  @override
  Future<int> setCameraExposurePosition(double x, double y) async {
    int reply = await _api.setCameraExposurePosition(SetCameraPositionRequest()
      ..x = x
      ..y = y);
    return reply;
  }

  @override
  Future<bool> isSpeakerphoneOn() async {
    bool reply = await _api.isSpeakerphoneOn();
    return reply;
  }

  @override
  Future<int> setSpeakerphoneOn(bool enable) async {
    int reply = await _api.setSpeakerphoneOn(enable);
    return reply;
  }

  @override
  Future<int> enableEarback(bool enabled, int volume) async {
    int reply = await _api.enableEarback(enabled, volume);
    return reply;
  }

  @override
  Future<int> setEarbackVolume(int volume) async {
    int reply = await _api.setEarbackVolume(volume);
    return reply;
  }

  @override
  Future<int> setAudioFocusMode(int focusMode) async {
    int reply = await _api.setAudioFocusMode(focusMode);
    return reply;
  }

  @override
  Future<int> getCameraCurrentZoom() async {
    int reply = await _api.getCameraCurrentZoom();
    return reply;
  }

  @override
  Future<int> getCurrentCamera() async {
    int reply = await _api.getCurrentCamera();
    return reply;
  }

  @override
  Future<int> switchCameraWithPosition(int position) async {
    int reply = await _api.switchCameraWithPosition(position);
    return reply;
  }
}

class MethodChannelRtcDeviceEventSink extends NERtcDeviceEventSink {
  final _listeners = <NERtcDeviceEventSink>[];

  void addRtcDeviceListener(NERtcDeviceEventSink sink) {
    _listeners.add(sink);
  }

  void removeRtcDeviceListener(NERtcDeviceEventSink sink) {
    _listeners.remove(sink);
  }

  @override
  void onAudioDeviceChanged(int selected) {
    for (var element in _listeners) {
      element.onAudioDeviceChanged(selected);
    }
  }

  @override
  void onAudioDeviceStateChange(int deviceType, int deviceState) {
    for (var element in _listeners) {
      element.onAudioDeviceStateChange(deviceType, deviceState);
    }
  }

  @override
  void onVideoDeviceStateChange(int deviceType, int deviceState) {
    for (var element in _listeners) {
      element.onVideoDeviceStateChange(deviceType, deviceState);
    }
  }

  @override
  void onCameraExposureChanged(CGPoint exposurePoint) {
    for (var element in _listeners) {
      element.onCameraExposureChanged(exposurePoint);
    }
  }

  @override
  void onCameraFocusChanged(CGPoint focusPoint) {
    for (var element in _listeners) {
      element.onCameraFocusChanged(focusPoint);
    }
  }
}
