// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:nertc_core_platform_interface/pigeon.dart';
import '../platform_interface/nertc_stats_platform_interface.dart';

class MethodChannelNERtcStatsSink extends NERtcStatsEventSink {
  final _listeners = <NERtcStatsEventSink>{};

  void registerStatsListener(NERtcStatsEventSink listener) {
    _listeners.add(listener);
  }

  void unregisterStatsListener(NERtcStatsEventSink listener) {
    _listeners.remove(listener);
  }

  @override
  void onLocalAudioStats(Map<dynamic, dynamic> arguments) {
    for (var element in _listeners) {
      element.onLocalAudioStats(arguments);
    }
  }

  @override
  void onLocalVideoStats(Map<dynamic, dynamic> arguments) {
    for (var element in _listeners) {
      element.onLocalVideoStats(arguments);
    }
  }

  @override
  void onNetworkQuality(Map<dynamic, dynamic> arguments) {
    for (var element in _listeners) {
      element.onNetworkQuality(arguments);
    }
  }

  @override
  void onRemoteAudioStats(Map<dynamic, dynamic> arguments) {
    for (var element in _listeners) {
      element.onRemoteAudioStats(arguments);
    }
  }

  @override
  void onRemoteVideoStats(Map<dynamic, dynamic> arguments) {
    for (var element in _listeners) {
      element.onRemoteVideoStats(arguments);
    }
  }

  @override
  void onRtcStats(Map<dynamic, dynamic> arguments) {
    for (var element in _listeners) {
      element.onRtcStats(arguments);
    }
  }
}

class MethodChannelNERtcStats extends NERtcStatsPlatform {
  final _eventDispatcher = MethodChannelNERtcStatsSink();

  MethodChannelNERtcStats() {
    NERtcStatsEventSink.setup(_eventDispatcher);
  }

  void registerStatsListener(NERtcStatsEventSink listener) {
    _eventDispatcher.registerStatsListener(listener);
  }

  void unregisterStatsListener(NERtcStatsEventSink listener) {
    _eventDispatcher.registerStatsListener(listener);
  }
}

class MethodChannelNERtcLiveStreamSink extends NERtcLiveStreamEventSink {
  final _listeners = <NERtcLiveStreamEventSink>{};

  void registerLiveStreamListener(NERtcLiveStreamEventSink listener) {
    _listeners.add(listener);
  }

  void unregisterLiveStreamListener(NERtcLiveStreamEventSink listener) {
    _listeners.remove(listener);
  }

  @override
  void onAddLiveStreamTask(String taskId, int errCode) {
    for (var element in _listeners) {
      element.onAddLiveStreamTask(taskId, errCode);
    }
  }

  @override
  void onDeleteLiveStreamTask(String taskId, int errCode) {
    for (var element in _listeners) {
      element.onDeleteLiveStreamTask(taskId, errCode);
    }
  }

  @override
  void onUpdateLiveStreamTask(String taskId, int errCode) {
    for (var element in _listeners) {
      element.onUpdateLiveStreamTask(taskId, errCode);
    }
  }
}

class MethodChannelNERtcLiveStream extends NERtcLiveStreamPlatform {
  final _eventDispatcher = MethodChannelNERtcLiveStreamSink();

  MethodChannelNERtcLiveStream() {
    NERtcLiveStreamEventSink.setup(_eventDispatcher);
  }

  @override
  void registerLiveStreamListener(NERtcLiveStreamEventSink listener) {
    _eventDispatcher.registerLiveStreamListener(listener);
  }

  @override
  void unregisterLiveStreamListener(NERtcLiveStreamEventSink listener) {
    _eventDispatcher.unregisterLiveStreamListener(listener);
  }
}
