// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:plugin_platform_interface/plugin_platform_interface.dart';
import 'package:nertc_core_platform_interface/method_channel/nertc_audio_effect_method_channel.dart';
import 'package:nertc_core_platform_interface/pigeon.dart';

abstract class NERtcAudioEffectPlatform extends PlatformInterface {
  NERtcAudioEffectPlatform() : super(token: _token);

  static final Object _token = Object();

  static NERtcAudioEffectPlatform _instance = MethodChannelNERtcAudioEffect();

  static NERtcAudioEffectPlatform get instance => _instance;

  static set instance(NERtcAudioEffectPlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  void addRtcAudioEffectListener(NERtcAudioEffectEventSink sink);

  void removeRtcAudioEffectListener(NERtcAudioEffectEventSink sink);

  Future<int> playEffect(int effectId, NERtcAudioEffectOptions options);

  Future<int> stopEffect(int effectId);

  Future<int> stopAllEffects();

  Future<int> pauseEffect(int effectId);

  Future<int> pauseAllEffect();

  Future<int> resumeEffect(int effectId);

  Future<int> resumeAllEffect();

  Future<int> setEffectSendVolume(int effectId, int volume);

  Future<int> getEffectSendVolume(int effectId);

  Future<int> setEffectPlaybackVolume(int effectId, int volume);

  Future<int> getEffectPlaybackVolume(int effectId);

  Future<int> getEffectDuration(int effectId);

  Future<int> getEffectCurrentPosition(int effectId);

  Future<int> setEffectPitch(int effectId, int pitch);

  Future<int> getEffectPitch(int effectId);

  Future<int> setEffectPosition(int effectId, int position);
}

/// 音效构造参数类
class NERtcAudioEffectOptions {
  const NERtcAudioEffectOptions(
      {required this.path,
      this.loopCount = 1,
      this.sendEnabled = true,
      this.sendVolume = 100,
      this.playbackEnabled = true,
      this.playbackVolume = 100,
      this.progressInterval = 1000,
      this.sendWithAudioType = 0,
      this.startTimestamp = 0});

  /// 文件地址
  final String path;

  /// 循环次数， <= 0, 表示无限循环，默认 1
  final int loopCount;

  /// 是否发送（默认 true）
  final bool sendEnabled;

  /// 发送音量[0 - 100]（默认 100）
  final int sendVolume;

  /// 是否播放（默认 true）
  final bool playbackEnabled;

  /// 播放音量[0 - 100]（默认 100）
  final int playbackVolume;

  /// 音效文件开始播放的时间，UTC 时间戳，默认值为 0，表示立即播放。
  final int startTimestamp;

  /// 音效跟随音频主流还是辅流，默认跟随主流,详细信息参考[NERtcAudioStreamType]
  final int sendWithAudioType;

  /// 音效播放进度回调间隔，单位ms，取值范围为 100~10000, 默认1000ms
  final int progressInterval;

  @override
  String toString() {
    return 'NERtcAudioEffectOptions{path:$path,'
        'loopCount:$loopCount,sendEnabled:$sendEnabled,sendVolume:$sendVolume,'
        'playbackEnabled:$playbackEnabled,playbackVolume:$playbackVolume,'
        'startTimestamp:$startTimestamp,sendWithAudioType:$sendWithAudioType,progressInterval:$progressInterval}';
  }
}
