// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:nertc_core_platform_interface/nertc_core_platform_interface.dart';
import 'package:plugin_platform_interface/plugin_platform_interface.dart';

abstract class NERtcAudioMixingPlatform extends PlatformInterface {
  NERtcAudioMixingPlatform() : super(token: _token);

  static final Object _token = Object();

  static NERtcAudioMixingPlatform _instance = MethodChannelNERtcAudioMixing();

  static NERtcAudioMixingPlatform get instance => _instance;

  static set instance(NERtcAudioMixingPlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  void addRtcAudioMixingListener(NERtcAudioMixingEventSink sink);

  void removeRtcAudioMixingListener(NERtcAudioMixingEventSink sink);

  Future<int> startAudioMixing(NERtcAudioMixingOptions options);

  Future<int> stopAudioMixing();

  Future<int> pauseAudioMixing();

  Future<int> resumeAudioMixing();

  Future<int> setAudioMixingSendVolume(int volume);

  Future<int> getAudioMixingSendVolume();

  Future<int> setAudioMixingPlaybackVolume(int volume);

  Future<int> getAudioMixingPlaybackVolume();

  Future<int> getAudioMixingDuration();

  Future<int> getAudioMixingCurrentPosition();

  Future<int> setAudioMixingPosition(int position);

  Future<int> setAudioMixingPitch(int pitch);

  Future<int> getAudioMixingPitch();
}

/// 伴音构造参数类
class NERtcAudioMixingOptions {
  NERtcAudioMixingOptions(
      {required this.path,
      this.loopCount = 1,
      this.sendEnabled = true,
      this.sendVolume = 100,
      this.playbackEnabled = true,
      this.playbackVolume = 100,
      this.progressInterval = 1000,
      this.sendWithAudioType = 0,
      this.startTimeStamp = 0});

  /// 文件地址/URL
  String path;

  /// 循环次数， <= 0, 表示无限循环，默认 1
  int loopCount;

  /// 是否发送（默认 true）
  bool sendEnabled;

  /// 发送音量[0 - 100]（默认 100）
  int sendVolume;

  /// 是否播放（默认 true）
  bool playbackEnabled;

  /// 音乐文件的播放音量，取值范围为 0~200。默认为 100，表示使用文件的原始音量
  int playbackVolume;

  ///音乐文件开始播放的时间，UTC 时间戳，即从1970 年 1 月 1 日 0 点 0 分 0 秒开始到事件发生时的毫秒数。默认值为 0，表示立即播放
  int startTimeStamp;

  ///伴音跟随音频主流还是辅流，默认跟随主流, 类型参考 [NERtcAudioStreamType]
  int sendWithAudioType;

  ///播放进度回调间隔，单位ms，取值范围为 100~10000, 默认1000ms
  int progressInterval;

  @override
  String toString() {
    return 'NERtcAudioMixingOptions{path:$path,'
        'loopCount:$loopCount,sendEnabled:$sendEnabled,sendVolume:$sendVolume,'
        'playbackEnabled:$playbackEnabled,playbackVolume:$playbackVolume,'
        'progressInterval:$progressInterval,sendWithAudioType:$sendWithAudioType,startTimeStamp:$startTimeStamp}';
  }
}
