// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:pigeon/pigeon.dart';

/// onUserJoined 回调的一些可选信息
class NERtcUserJoinExtraInfo {
  ///自定义信息， 来源于远端用户joinChannel时填的 [NERtcJoinChannelOptions.customInfo]参数，默认为空字符串。
  String customInfo;
  NERtcUserJoinExtraInfo(this.customInfo);
}

/// onUserLeave 回调的一些可选信息
class NERtcUserLeaveExtraInfo {
  /// 自定义信息, 来源于远端用户joinChannel时填的 [NERtcJoinChannelOptions.customInfo]参数，默认为空字符串。
  String customInfo;
  NERtcUserLeaveExtraInfo(this.customInfo);
}

class UserJoinedEvent {
  int uid;
  NERtcUserJoinExtraInfo? joinExtraInfo;
  UserJoinedEvent(this.uid, this.joinExtraInfo);
}

class UserLeaveEvent {
  int uid;
  int reason;
  NERtcUserLeaveExtraInfo? leaveExtraInfo;
  UserLeaveEvent(this.uid, this.reason, this.leaveExtraInfo);
}

class UserVideoMuteEvent {
  int uid;
  bool muted;
  int? streamType;
  UserVideoMuteEvent(this.uid, this.muted, this.streamType);
}

class FirstVideoDataReceivedEvent {
  int uid;
  int? streamType;
  FirstVideoDataReceivedEvent(this.uid, this.streamType);
}

class FirstVideoFrameDecodedEvent {
  int uid;
  int width;
  int height;
  int? streamType;
  FirstVideoFrameDecodedEvent(
      this.uid, this.width, this.height, this.streamType);
}

class VirtualBackgroundSourceEnabledEvent {
  final bool enabled;
  final int reason;
  VirtualBackgroundSourceEnabledEvent(this.enabled, this.reason);
}

class AudioVolumeInfo {
  int uid;
  int volume;
  int subStreamVolume;
  AudioVolumeInfo(this.uid, this.volume, this.subStreamVolume);
}

class RemoteAudioVolumeIndicationEvent {
  List<AudioVolumeInfo?>? volumeList;
  int totalVolume;
  RemoteAudioVolumeIndicationEvent(this.volumeList, this.totalVolume);
}

///上下行 Last mile 网络质量探测结果
class NERtcLastmileProbeResult {
  ///Last mile 质量探测结果的状态, 详细参数见 [NERtcLastmileProbeResultState]
  int state;

  ///往返时延，单位为毫秒(ms)
  int rtt;

  ///上行网络质量报告
  NERtcLastmileProbeOneWayResult uplinkReport;

  ///下行网络质量报告
  NERtcLastmileProbeOneWayResult downlinkReport;

  NERtcLastmileProbeResult(
      this.state, this.rtt, this.uplinkReport, this.downlinkReport);
}

///单向 Last mile 网络质量探测结果报告
class NERtcLastmileProbeOneWayResult {
  ///丢包率
  int packetLossRate;

  ///网络抖动，单位为毫秒 (ms)
  int jitter;

  ///可用网络带宽预估，单位为 bps
  int availableBandwidth;

  NERtcLastmileProbeOneWayResult(
      this.packetLossRate, this.jitter, this.availableBandwidth);
}

@FlutterApi()
abstract class NERtcChannelEventSink {
  void onJoinChannel(int result, int channelId, int elapsed, int uid);

  void onLeaveChannel(int result);

  void onUserJoined(UserJoinedEvent event);

  void onUserLeave(UserLeaveEvent event);

  void onUserAudioStart(int uid);

  void onUserSubStreamAudioStart(int uid);

  void onUserAudioStop(int uid);

  void onUserSubStreamAudioStop(int uid);

  void onUserVideoStart(int uid, int maxProfile);

  void onUserVideoStop(int uid);

  void onDisconnect(int reason);

  void onUserAudioMute(int uid, bool muted);

  void onUserVideoMute(UserVideoMuteEvent event);

  void onUserSubStreamAudioMute(int uid, bool muted);

  void onFirstAudioDataReceived(int uid);

  void onFirstVideoDataReceived(FirstVideoDataReceivedEvent event);

  void onFirstAudioFrameDecoded(int uid);

  void onFirstVideoFrameDecoded(FirstVideoFrameDecodedEvent event);

  void onVirtualBackgroundSourceEnabled(
      VirtualBackgroundSourceEnabledEvent event);

  void onConnectionTypeChanged(int newConnectionType);

  void onReconnectingStart();

  void onReJoinChannel(int result, int channelId);

  void onConnectionStateChanged(int state, int reason);

  void onLocalAudioVolumeIndication(int volume, bool vadFlag);

  void onRemoteAudioVolumeIndication(RemoteAudioVolumeIndicationEvent event);

  void onLiveStreamState(String taskId, String pushUrl, int liveState);

  void onClientRoleChange(int oldRole, int newRole);

  void onError(int code);

  void onWarning(int code);

  void onUserSubStreamVideoStart(int uid, int maxProfile);

  void onUserSubStreamVideoStop(int uid);

  void onAudioHasHowling();

  void onRecvSEIMsg(int userID, String seiMsg);

  void onAudioRecording(int code, String filePath);

  void onMediaRightChange(
      bool isAudioBannedByServer, bool isVideoBannedByServer);

  void onMediaRelayStatesChange(int state, String channelName);

  void onMediaRelayReceiveEvent(int event, int code, String channelName);

  void onLocalPublishFallbackToAudioOnly(bool isFallback, int streamType);

  void onRemoteSubscribeFallbackToAudioOnly(
      int uid, bool isFallback, int streamType);

  void onLocalVideoWatermarkState(int videoStreamType, int state);

  void onLastmileQuality(int quality);

  void onLastmileProbeResult(NERtcLastmileProbeResult result);

  void onTakeSnapshotResult(int code, String path);
}

class RtcServerAddresses {
  bool? valid;
  String? channelServer;
  String? statisticsServer;
  String? roomServer;
  String? compatServer;
  String? nosLbsServer;
  String? nosUploadSever;
  String? nosTokenServer;
  String? sdkConfigServer;
  String? cloudProxyServer;
  String? webSocketProxyServer;
  String? quicProxyServer;
  String? mediaProxyServer;
  String? statisticsDispatchServer;
  String? statisticsBackupServer;
  bool? useIPv6;
}

class CreateEngineRequest {
  String? appKey;
  String? logDir;
  RtcServerAddresses? serverAddresses;
  int? logLevel;
  bool? audioAutoSubscribe;
  bool? videoAutoSubscribe;
  bool? disableFirstJoinUserCreateChannel;
  bool? audioDisableOverrideSpeakerOnReceiver;
  bool? audioDisableSWAECOnHeadset;
  bool? audioAINSEnabled;
  bool? serverRecordAudio;
  bool? serverRecordVideo;
  int? serverRecordMode;
  bool? serverRecordSpeaker;
  bool? publishSelfStream;
  bool? videoCaptureObserverEnabled;
  int? videoEncodeMode;
  int? videoDecodeMode;
  int? videoSendMode;
  bool? videoH265Enabled;
  bool? mode1v1Enabled;
}

class JoinChannelOptions {
  String? customInfo;
  String? permissionKey;
}

class JoinChannelRequest {
  String? token;
  String channelName;
  int uid;
  JoinChannelOptions? channelOptions;
  JoinChannelRequest(this.uid, this.channelName);
}

class SubscribeRemoteAudioRequest {
  int? uid;
  bool? subscribe;
}

class EnableLocalVideoRequest {
  bool? enable;
  int? streamType;
}

class SetAudioProfileRequest {
  int? profile;
  int? scenario;
}

class SetLocalVideoConfigRequest {
  int? videoProfile;
  int? videoCropMode;
  bool? frontCamera;
  int? frameRate;
  int? minFrameRate;
  int? bitrate;
  int? minBitrate;
  int? degradationPrefer;
  int? width;
  int? height;
  int? cameraType;
  int? mirrorMode;
  int? orientationMode;
  int? streamType;
}

/// 摄像头采集配置
class SetCameraCaptureConfigRequest {
  /// 设置摄像头的额外旋转信息
  NERtcCaptureExtraRotation? extraRotation;

  /// 本地采集的视频宽度，单位为 px。
  ///
  /// 视频编码分辨率以宽 x 高表示，用于设置视频编码分辨率，以衡量编码质量。
  ///
  ///captureWidth 表示视频帧在横轴上的像素，即自定义宽。
  int? captureWidth;

  /// 本地采集的视频宽度，单位为 px。
  ///
  /// 视频编码分辨率以宽 x 高表示，用于设置视频编码分辨率，以衡量编码质量。
  ///
  ///captureHeight 表示视频帧在横轴上的像素，即自定义高。
  int? captureHeight;

  /// 视频流类型
  int? streamType;
}

class StartorStopVideoPreviewRequest {
  int? streamType;
}

class StartScreenCaptureRequest {
  int? contentPrefer;
  int? videoProfile;
  int? frameRate;
  int? minFrameRate;
  int? bitrate;
  int? minBitrate;
}

class SubscribeRemoteVideoStreamRequest {
  int? uid;
  int? streamType;
  bool? subscribe;
}

class SubscribeRemoteSubStreamVideoRequest {
  int? uid;
  bool? subscribe;
}

// class SetupRemoteVideoRendererRequest {
//   int? uid;
//   int? textureId;
// }

// class SetupRemoteSubStreamVideoRendererRequest {
//   int? uid;
//   int? textureId;
// }

class EnableAudioVolumeIndicationRequest {
  bool? enable;
  int? interval;
  bool? vad;
}

class SubscribeRemoteSubStreamAudioRequest {
  bool? subscribe;
  int? uid;
}

class SetAudioSubscribeOnlyByRequest {
  List<int?>? uidArray;
}

class StartAudioMixingRequest {
  String? path;
  int? loopCount;
  bool? sendEnabled;
  int? sendVolume;
  bool? playbackEnabled;
  int? playbackVolume;
  int? startTimeStamp;
  int? sendWithAudioType;
  int? progressInterval;
}

class PlayEffectRequest {
  int? effectId;
  String? path;
  int? loopCount;
  bool? sendEnabled;
  int? sendVolume;
  bool? playbackEnabled;
  int? playbackVolume;
  int? startTimestamp;
  int? sendWithAudioType;
  int? progressInterval;
}

class SetEffectSendVolumeRequest {
  int? effectId;
  int? volume;
}

class SetEffectPlaybackVolumeRequest {
  int? effectId;
  int? volume;
}

class SetCameraPositionRequest {
  double? x;
  double? y;
}

class EnableEarbackRequest {
  bool? enabled;
  int? volume;
}

class AddOrUpdateLiveStreamTaskRequest {
  int? serial;
  String? taskId;
  String? url;
  bool? serverRecordEnabled;
  int? liveMode;
  int? layoutWidth;
  int? layoutHeight;
  int? layoutBackgroundColor;
  String? layoutImageUrl;
  int? layoutImageX;
  int? layoutImageY;
  int? layoutImageWidth;
  int? layoutImageHeight;
  List? layoutUserTranscodingList;
}

class DeleteLiveStreamTaskRequest {
  int? serial;
  String? taskId;
}

class SetVideoRendererMirrorRequest {
  int? textureId;
  bool? mirror;
}

class SendSEIMsgRequest {
  String? seiMsg;
  int? streamType;
}

class ListValue {
  List? list;
}

class SetLocalVoiceEqualizationRequest {
  int? bandFrequency;
  int? bandGain;
}

class SwitchChannelRequest {
  String? token;
  String? channelName;
  JoinChannelOptions? channelOptions;
}

class StartAudioRecordingRequest {
  String? filePath;
  int? sampleRate;
  int? quality;
}

class AudioRecordingConfigurationRequest {
  String? filePath;
  int? sampleRate;
  int? quality;
  int? position;
  int? cycleTime;
}

class SetLocalMediaPriorityRequest {
  int? priority;
  bool? isPreemptive;
}

class StartOrUpdateChannelMediaRelayRequest {
  Map<Object?, Object?>? sourceMediaInfo;
  Map<Object?, Map<Object?, Object?>?>? destMediaInfo;
}

class AdjustUserPlaybackSignalVolumeRequest {
  int? uid;
  int? volume;
}

class EnableEncryptionRequest {
  String? key;
  int? mode;
  bool? enable;
}

class SetLocalVoiceReverbParamRequest {
  double? wetGain;
  double? dryGain;
  double? damping;
  double? roomSize;
  double? decayTime;
  double? preDelay;
}

class ReportCustomEventRequest {
  String? eventName;
  String? customIdentify;
  Map<String?, Object?>? param;
}

class StartLastmileProbeTestRequest {
  bool? probeUplink;
  bool? probeDownlink;
  int? expectedUplinkBitrate;
  int? expectedDownlinkBitrate;
}

/// 顶点坐标
class CGPoint {
  /// x 的 取值范围是 &#91; 0-1 &#93;
  double x;

  /// y 的 取值范围是 &#91; 0-1 &#93;
  double y;
  CGPoint(this.x, this.y);
}

class SetVideoCorrectionConfigRequest {
  CGPoint? topLeft;
  CGPoint? topRight;
  CGPoint? bottomLeft;
  CGPoint? bottomRight;
  double? canvasWidth;
  double? canvasHeight;
  bool? enableMirror;
}

class EnableVirtualBackgroundRequest {
  bool? enabled;
  int? backgroundSourceType;
  int? color;
  String? source;
  int? blur_degree;
}

class SetRemoteHighPriorityAudioStreamRequest {
  bool? enabled;
  int? uid;
  int? streamType;
}

/// 视频水印类型
enum NERtcVideoWatermarkType {
  ///图片
  kNERtcVideoWatermarkTypeImage,

  ///文字
  kNERtcVideoWatermarkTypeText,

  ///时间戳
  kNERtcVideoWatermarkTypeTimeStamp,
}

class VideoWatermarkImageConfig {
  double? wmAlpha;
  int? wmWidth;
  int? wmHeight;
  int? offsetX;
  int? offsetY;
  List<String?>? imagePaths;
  int? fps;
  bool? loop;
}

class VideoWatermarkTextConfig {
  double? wmAlpha;
  int? wmWidth;
  int? wmHeight;
  int? offsetX;
  int? offsetY;
  int? wmColor;
  int? fontSize;
  int? fontColor;
  String? fontNameOrPath;
  String? content;
}

class VideoWatermarkTimestampConfig {
  double? wmAlpha;
  int? wmWidth;
  int? wmHeight;
  int? offsetX;
  int? offsetY;
  int? wmColor;
  int? fontSize;
  int? fontColor;
  String? fontNameOrPath;
}

///视频水印设置，目前支持三种类型的水印，但只能其中选择一种水印生效
class VideoWatermarkConfig {
  NERtcVideoWatermarkType WatermarkType;
  VideoWatermarkImageConfig? imageWatermark;
  VideoWatermarkTextConfig? textWatermark;
  VideoWatermarkTimestampConfig? timestampWatermark;
  VideoWatermarkConfig(this.WatermarkType, this.imageWatermark,
      this.textWatermark, this.timestampWatermark);
}

class SetLocalVideoWatermarkConfigsRequest {
  int type;
  VideoWatermarkConfig? config;
  SetLocalVideoWatermarkConfigsRequest(this.type, this.config);
}

///摄像头额外旋转信息
enum NERtcCaptureExtraRotation {
  ///（默认）没有额外的旋转信息，直接使用系统旋转参数处理
  kNERtcCaptureExtraRotationDefault,

  ///在系统旋转信息的基础上，额外顺时针旋转90度
  kNERtcCaptureExtraRotationClockWise90,

  ///在系统旋转信息的基础上，额外旋转180度
  kNERtcCaptureExtraRotation180,

  ///在系统旋转信息的基础上，额外逆时针旋转90度
  kNERtcCaptureExtraRotationAntiClockWise90,
}

/// NERtc 版本信息
class NERtcVersion {
  String? versionName;
  int? versionCode;
  String? buildType;
  String? buildDate;
  String? buildRevision;
  String? buildHost;
  String? serverEnv;
  String? buildBranch;
  String? engineRevision;
}

class VideoFrame {
  int? width;
  int? height;
  int? rotation;
  int? format;
  int? timeStamp;
  Uint8List? data;
  int? strideY;
  int? strideU;
  int? strideV;
  int? textureId;
  List<double?>? transformMatrix;
}

@HostApi()
abstract class EngineApi {
  int create(CreateEngineRequest request);

  NERtcVersion version();

  List<String>? checkPermission();

  int setParameters(Map<String, Object> params);

  @async
  int release();

  int setStatsEventCallback();

  int clearStatsEventCallback();

  int setChannelProfile(int channelProfile);

  int joinChannel(JoinChannelRequest request);

  int leaveChannel();

  int updatePermissionKey(String key);

  int enableLocalAudio(bool enable);

  int subscribeRemoteAudio(SubscribeRemoteAudioRequest request);

  int subscribeAllRemoteAudio(bool subscribe);

  int setAudioProfile(SetAudioProfileRequest request);

  int enableDualStreamMode(bool enable);

  int setLocalVideoConfig(SetLocalVideoConfigRequest request);

  int setCameraCaptureConfig(SetCameraCaptureConfigRequest request);

  int startVideoPreview(StartorStopVideoPreviewRequest request);

  int stopVideoPreview(StartorStopVideoPreviewRequest request);

  int enableLocalVideo(EnableLocalVideoRequest request);

  int enableLocalSubStreamAudio(bool enable);

  int subscribeRemoteSubStreamAudio(
      SubscribeRemoteSubStreamAudioRequest request);

  int muteLocalSubStreamAudio(bool muted);

  int setAudioSubscribeOnlyBy(SetAudioSubscribeOnlyByRequest request);

  @async
  int startScreenCapture(StartScreenCaptureRequest request);

  int stopScreenCapture();

  int enableLoopbackRecording(bool enable);

  int subscribeRemoteVideoStream(SubscribeRemoteVideoStreamRequest request);

  int subscribeRemoteSubStreamVideo(
      SubscribeRemoteSubStreamVideoRequest request);

  int muteLocalAudioStream(bool mute);

  int muteLocalVideoStream(bool mute, int streamType);

  int startAudioDump();

  int startAudioDumpWithType(int dumpType);

  int stopAudioDump();

  int enableAudioVolumeIndication(EnableAudioVolumeIndicationRequest request);

  int adjustRecordingSignalVolume(int volume);

  int adjustPlaybackSignalVolume(int volume);

  int adjustLoopBackRecordingSignalVolume(int volume);

  int addLiveStreamTask(AddOrUpdateLiveStreamTaskRequest request);

  int updateLiveStreamTask(AddOrUpdateLiveStreamTaskRequest request);

  int removeLiveStreamTask(DeleteLiveStreamTaskRequest request);

  int setClientRole(int role);

  int getConnectionState();

  int uploadSdkInfo();

  int sendSEIMsg(SendSEIMsgRequest request);

  int setLocalVoiceReverbParam(SetLocalVoiceReverbParamRequest request);

  int setAudioEffectPreset(int preset);

  int setVoiceBeautifierPreset(int preset);

  int setLocalVoicePitch(double pitch);

  int setLocalVoiceEqualization(SetLocalVoiceEqualizationRequest request);

  int switchChannel(SwitchChannelRequest request);

  int startAudioRecording(StartAudioRecordingRequest request);

  int startAudioRecordingWithConfig(AudioRecordingConfigurationRequest request);

  int stopAudioRecording();

  int setLocalMediaPriority(SetLocalMediaPriorityRequest request);

  int enableMediaPub(int mediaType, bool enable);

  int startChannelMediaRelay(StartOrUpdateChannelMediaRelayRequest request);
  int updateChannelMediaRelay(StartOrUpdateChannelMediaRelayRequest request);
  int stopChannelMediaRelay();

  int adjustUserPlaybackSignalVolume(
      AdjustUserPlaybackSignalVolumeRequest request);

  int setLocalPublishFallbackOption(int option);
  int setRemoteSubscribeFallbackOption(int option);

  int enableSuperResolution(bool enable);
  int enableEncryption(EnableEncryptionRequest request);

  int setAudioSessionOperationRestriction(int option);

  int enableVideoCorrection(bool enable);

  int reportCustomEvent(ReportCustomEventRequest request);

  int getEffectDuration(int effectId);

  // int getEffectCurrentPosition(int effectId);

  int startLastmileProbeTest(StartLastmileProbeTestRequest request);

  int stopLastmileProbeTest();

  int setVideoCorrectionConfig(SetVideoCorrectionConfigRequest request);

  int enableVirtualBackground(EnableVirtualBackgroundRequest request);

  int setRemoteHighPriorityAudioStream(
      SetRemoteHighPriorityAudioStreamRequest request);

  int setCloudProxy(int proxyType);

  int startBeauty();

  void stopBeauty();

  int enableBeauty(bool enabled);

  int setBeautyEffect(double level, int beautyType);

  int addBeautyFilter(String path, String name);

  void removeBeautyFilter();

  int setBeautyFilterLevel(double level);

  int setLocalVideoWatermarkConfigs(
      SetLocalVideoWatermarkConfigsRequest request);

  int setStreamAlignmentProperty(bool enable);

  int getNtpTimeOffset();

  int takeLocalSnapshot(int streamType, String path);

  int takeRemoteSnapshot(int uid, int streamType, String path);

  int setExternalVideoSource(int streamType, bool enable);

  int pushExternalVideoFrame(int streamType, VideoFrame frame);
}

@HostApi()
abstract class VideoRendererApi {
  int createVideoRenderer();

  int setMirror(int textureId, bool mirror);

  int setupLocalVideoRenderer(int textureId);

  int setupRemoteVideoRenderer(int uid, int textureId);

  int setupLocalSubStreamVideoRenderer(int textureId);

  int setupRemoteSubStreamVideoRenderer(int uid, int textureId);

  void disposeVideoRenderer(int textureId);
}

@FlutterApi()
abstract class NERtcDeviceEventSink {
  void onAudioDeviceChanged(int selected);

  void onAudioDeviceStateChange(int deviceType, int deviceState);

  void onVideoDeviceStateChange(int deviceType, int deviceState);

  void onCameraFocusChanged(CGPoint focusPoint);

  void onCameraExposureChanged(CGPoint exposurePoint);
}

@HostApi()
abstract class DeviceManagerApi {
  bool isSpeakerphoneOn();

  bool isCameraZoomSupported();

  bool isCameraTorchSupported();

  bool isCameraFocusSupported();

  bool isCameraExposurePositionSupported();

  int setSpeakerphoneOn(bool enable);

  int switchCamera();

  int setCameraZoomFactor(double factor);

  double getCameraMaxZoom();

  int setCameraTorchOn(bool on);

  int setCameraFocusPosition(SetCameraPositionRequest request);

  int setCameraExposurePosition(SetCameraPositionRequest request);

  int setPlayoutDeviceMute(bool mute);

  bool isPlayoutDeviceMute();

  int setRecordDeviceMute(bool mute);

  bool isRecordDeviceMute();

  int enableEarback(bool enabled, int volume);

  int setEarbackVolume(int volume);

  int setAudioFocusMode(int focusMode);

  int getCurrentCamera();

  int switchCameraWithPosition(int position);

  int getCameraCurrentZoom();
}

@HostApi()
abstract class AudioMixingApi {
  int startAudioMixing(StartAudioMixingRequest request);

  int stopAudioMixing();

  int pauseAudioMixing();

  int resumeAudioMixing();

  int setAudioMixingSendVolume(int volume);

  int getAudioMixingSendVolume();

  int setAudioMixingPlaybackVolume(int volume);

  int getAudioMixingPlaybackVolume();

  int getAudioMixingDuration();

  int getAudioMixingCurrentPosition();

  int setAudioMixingPosition(int position);

  int setAudioMixingPitch(int pitch);

  int getAudioMixingPitch();
}

@FlutterApi()
abstract class NERtcAudioMixingEventSink {
  void onAudioMixingStateChanged(int reason);

  void onAudioMixingTimestampUpdate(int timestampMs);
}

@HostApi()
abstract class AudioEffectApi {
  int playEffect(PlayEffectRequest request);

  int stopEffect(int effectId);

  int stopAllEffects();

  int pauseEffect(int effectId);

  int resumeEffect(int effectId);

  int pauseAllEffects();

  int resumeAllEffects();

  int setEffectSendVolume(int effectId, int volume);

  int getEffectSendVolume(int effectId);

  int setEffectPlaybackVolume(int effectId, int volume);

  int getEffectPlaybackVolume(int effectId);

  int getEffectDuration(int effectId);

  int getEffectCurrentPosition(int effectId);

  int setEffectPitch(int effectId, int pitch);

  int getEffectPitch(int effectId);

  int setEffectPosition(int effectId, int position);
}

@FlutterApi()
abstract class NERtcAudioEffectEventSink {
  void onAudioEffectFinished(int effectId);

  void onAudioEffectTimestampUpdate(int id, int timestampMs);
}

@FlutterApi()
abstract class NERtcStatsEventSink {
  void onRtcStats(Map<Object, Object> arguments);

  void onLocalAudioStats(Map<Object, Object> arguments);

  void onRemoteAudioStats(Map<Object, Object> arguments);

  void onLocalVideoStats(Map<Object, Object> arguments);

  void onRemoteVideoStats(Map<Object, Object> arguments);

  void onNetworkQuality(Map<Object, Object> arguments);
}

@FlutterApi()
abstract class NERtcLiveStreamEventSink {
  void onUpdateLiveStreamTask(String taskId, int errCode);

  void onAddLiveStreamTask(String taskId, int errCode);

  void onDeleteLiveStreamTask(String taskId, int errCode);
}
